/*****************************************************************************/
/* XBalloon - Balloons move on root window.                                  */
/*                                                                           */
/* XBalloon Copyright (c) 1999 Sakai Hiroaki.                                */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* XBalloon is a simple demonstration program for X. Balloons move on root   */
/* window. You can use a faborite pixmap as balloons.                        */
/*                                                                           */
/* XBalloon is a free software. You may copy or distribute the original      */
/* XBalloon freely. But, you may not modify or distribute the original       */
/* XBalloon without permission of the author.                                */
/*****************************************************************************/

/*****************************************************************************/
/* Include Header Files.                                                     */
/*****************************************************************************/

#include "argument.h"

/*****************************************************************************/
/* Difinitions of Some Parameters.                                           */
/*****************************************************************************/

#define FLAG     0
#define FLAG_ON  1
#define FLAG_OFF 2
#define INTEGER  3
#define STRING   4
#define FUNCTION 5

/*****************************************************************************/
/* Help Message.                                                             */
/*****************************************************************************/

static char * help_string =
"\n"
VERSION "\n"
"\n"
"XBalloon - Balloons move on root window.\n"
"\n"
"XBalloon is a simple demonstration program for X. Balloons move on root\n"
"window. You can use a faborite pixmap as balloons.\n"
"\n"
"Options\n"
"    -h, -help               Output help message.\n"
"    -n, -number [n]         Number of Balloons.\n"
"    -speed [n]              Step length of balloons' motion.\n"
"    -velocity [n]           Fine motions of balloons.\n" 
"    -wait [n]               Wait.\n"
"    -display [displayname]  Displayname.\n"
"    -pixmap [filename]      Balloon's pixmap.\n"
"\n"
"XBalloon Copyright (c) 1999 Sakai Hiroaki.\n"
"All Rights Reserved.\n"
"\n"
"XBalloon is a free software. You may copy or distribute the original\n"
"XBalloon freely. But, you may not modify or distribute the original\n"
"XBalloon without permission of the author.\n"
"\n";

void Help()
{
  printf("%s", help_string);
  exit (0);
}

/*****************************************************************************/
/* Difinitions of Some Structures.                                           */
/*****************************************************************************/

struct {
  char * name;
  short int flag;
  void * value;
} arguments[] = {
  {"-h"        , FUNCTION, Help},
  {"-help"     , FUNCTION, Help},
  {"-n"        , INTEGER , &(balloons.number)},
  {"-number"   , INTEGER , &(balloons.number)},
  {"-speed"    , INTEGER , &(balloons.speed_maximum)},
  {"-velocity" , INTEGER , &(balloons.velocity_maximum)},
  {"-wait"     , INTEGER , &(xballoon.wait)},
  {"-display"  , STRING  , &(xballoon.display_name)},
  {"-pixmap"   , STRING  , &(xballoon.balloon_pixmap_filename)},
  {NULL        , 0       , NULL}
};

/*****************************************************************************/
/* Definitions of Functions.                                                 */
/*****************************************************************************/

void SetDefaultValue()
{
  xballoon.display_name = NULL;
  balloons.number = DEFAULT_NUMBER;
  balloons.speed_maximum = DEFAULT_SPEED_MAXIMUM;
  balloons.speed_minimum = DEFAULT_SPEED_MINIMUM;
  balloons.velocity_maximum = DEFAULT_VELOCITY_MAXIMUM;
  balloons.velocity_minimum = DEFAULT_VELOCITY_MINIMUM;
  xballoon.wait = DEFAULT_WAIT;
  xballoon.balloon_pixmap_filename = DEFAULT_BALLOON_PIXMAP_FILENAME;
}

void DeleteParameter(int * argc, char ** argv, int i)
{
  (*argc)--;
  for (; i < (*argc); i++)
    argv[i] = argv[i + 1];
}

int CheckParameters(int * argc, char ** argv, char * s)
{
  int i, r = OFF;

  for (i = 1; i < *argc; i++) {
    if (!strcmp(s, argv[i])) {
      DeleteParameter(argc, argv, i);
      r = ON;
    }
  }
  return (r);
}

char * GetStringFromParameters(int * argc, char ** argv, char * s)
{
  int i;
  char * p = (char *)NULL;

  for (i = 1; i < *argc; i++) {
    if (!strcmp(s, argv[i])) {
      if (i == *argc - 1) {
        DeleteParameter(argc, argv, i);
      } else {
        DeleteParameter(argc, argv, i);
        p = argv[i];
        DeleteParameter(argc, argv, i);
      }
    }
  }
  return (p);
}

void ReadParameters(int * argc, char ** argv)
{
  int i = 0;
  char * p;
  void (*funcp)();
  while (arguments[i].name != NULL) {
    if (arguments[i].flag == FUNCTION) {
      if (CheckParameters(argc, argv, arguments[i].name) == ON) {
        funcp = arguments[i].value;
        (*funcp)();
      }
    } else if (arguments[i].flag == FLAG) {
      if ((p = GetStringFromParameters(argc, argv, arguments[i].name))!=NULL) {
        if      (!strcmp(p, "off")) *((short int *)(arguments[i].value)) = OFF;
        else if (!strcmp(p, "OFF")) *((short int *)(arguments[i].value)) = OFF;
        else if (!strcmp(p, "on" )) *((short int *)(arguments[i].value)) = ON;
        else if (!strcmp(p, "ON" )) *((short int *)(arguments[i].value)) = ON;
      }
    } else if (arguments[i].flag == FLAG_ON) {
      if (CheckParameters(argc, argv, arguments[i].name) == ON)
        *((short int *)(arguments[i].value)) = ON;
    } else if (arguments[i].flag == FLAG_OFF) {
      if (CheckParameters(argc, argv, arguments[i].name) == ON)
        *((short int *)(arguments[i].value)) = OFF;
    } else if (arguments[i].flag == INTEGER) {
      if ((p = GetStringFromParameters(argc, argv, arguments[i].name)) != NULL)
        *((int *)(arguments[i].value)) = atoi(p);
    } else if (arguments[i].flag == STRING) {
      if ((p = GetStringFromParameters(argc, argv, arguments[i].name)) != NULL)
        *((char **)(arguments[i].value)) = p;
    }
    i++;        
  }
  while (*argc > 1) {
    fprintf(stderr, "ERROR : Unknown Parameter : %s\n", argv[1]);
    DeleteParameter(argc, argv, 1);
  }
}

/*****************************************************************************/
/* End of XBalloon.                                                          */
/*****************************************************************************/
