/*****************************************************************************/
/* XBalloon - Balloons move on root window.                                  */
/*                                                                           */
/* XBalloon Copyright (c) 1999 Sakai Hiroaki.                                */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* XBalloon is a simple demonstration program for X. Balloons move on root   */
/* window. You can use a faborite pixmap as balloons.                        */
/*                                                                           */
/* XBalloon is a free software. You may copy or distribute the original      */
/* XBalloon freely. But, you may not modify or distribute the original       */
/* XBalloon without permission of the author.                                */
/*****************************************************************************/

/*****************************************************************************/
/* Include Header Files.                                                     */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <X11/Xlib.h>
#include <X11/xpm.h>

#include "configure.h"
#include "xballoon.h"
#include "argument.h"
#include "display.h"
#include "graphic.h"
#include "init.h"

/*****************************************************************************/
/* Difinitions of Some Parameters.                                           */
/*****************************************************************************/

GC gc_copy, gc_and, gc_or, gc_xor, gc_and_inv;
Pixmap pixmap_work;

/*****************************************************************************/
/* Definitions of Functions.                                                 */
/*****************************************************************************/

void BalloonQuit()
{
  free(balloons.point);
  free(balloons.velocity);
  free(balloons.speed);
  free(balloons.count);
  XCloseDisplay(xballoon.display);
  exit (0);
}

void InterruptHandler()
{
  XClearWindow(xballoon.display, xballoon.root_window);
  BalloonQuit();
}

void InitializeSignal()
{
  signal(SIGINT, InterruptHandler);
}

void DrawPixmap1(XPoint old, XPoint new)
{
  XClearArea(xballoon.display, xballoon.root_window, old.x, old.y,
	     xballoon.balloon_width, xballoon.balloon_height, False);
  XCopyArea(xballoon.display, xballoon.balloon_figure,
	    xballoon.root_window, gc_and_inv,
	    0, 0, xballoon.balloon_width, xballoon.balloon_height,
	    new.x, new.y);
  XCopyArea(xballoon.display, xballoon.balloon_pixmap,
	    xballoon.root_window, gc_or, 0, 0,
	    xballoon.balloon_width, xballoon.balloon_height, new.x, new.y);
}

void DrawPixmap2(XPoint old, XPoint new)
{
  XCopyArea(xballoon.display, xballoon.balloon_pixmap, pixmap_work, gc_copy,
	    0, 0, xballoon.balloon_width, xballoon.balloon_height, 0, 0);
  XClearArea(xballoon.display, xballoon.root_window, old.x, old.y,
	     xballoon.balloon_width, xballoon.balloon_height, False);
  XCopyArea(xballoon.display, xballoon.root_window, pixmap_work, gc_xor,
	    new.x, new.y, xballoon.balloon_width, xballoon.balloon_height,
	    0, 0);
  XCopyArea(xballoon.display, xballoon.balloon_figure, pixmap_work, gc_and,
	    0, 0, xballoon.balloon_width, xballoon.balloon_height, 0, 0);
  XCopyArea(xballoon.display, pixmap_work, xballoon.root_window, gc_xor,
	    0, 0, xballoon.balloon_width, xballoon.balloon_height,
	    new.x, new.y);
}

void BalloonMain()
{
  int i;

  gc_copy    = XCreateGC(xballoon.display, xballoon.root_window, 0, NULL);
  gc_and     = XCreateGC(xballoon.display, xballoon.root_window, 0, NULL);
  gc_or      = XCreateGC(xballoon.display, xballoon.root_window, 0, NULL);
  gc_xor     = XCreateGC(xballoon.display, xballoon.root_window, 0, NULL);
  gc_and_inv = XCreateGC(xballoon.display, xballoon.root_window, 0, NULL);
  XSetFunction(xballoon.display, gc_copy   , GXcopy);
  XSetFunction(xballoon.display, gc_and    , GXand);
  XSetFunction(xballoon.display, gc_or     , GXor);
  XSetFunction(xballoon.display, gc_xor    , GXxor);
  XSetFunction(xballoon.display, gc_and_inv, GXandInverted);
  pixmap_work   = XCreatePixmap(xballoon.display, xballoon.root_window,
				xballoon.width, xballoon.height,
				xballoon.depth);
  while (1) {
    for (i = 0; i < balloons.number; i++) {
      balloons.point_old[i] = balloons.point[i];
      balloons.count[i] += balloons.speed[i];
      while (balloons.count[i] >= DEFAULT_COUNT_MAXIMUM) {
	balloons.count[i] -= DEFAULT_COUNT_MAXIMUM;
	balloons.point[i].x += balloons.velocity[i].x;
	balloons.point[i].y += balloons.velocity[i].y;
	if (balloons.point[i].x < 0) {
	  balloons.point[i].x *= -1;
	  balloons.velocity[i].x *= -1;
	} else if (balloons.point[i].x + xballoon.balloon_width >
		   xballoon.width) {
	  balloons.point[i].x -=
	    (balloons.point[i].x + xballoon.balloon_width - xballoon.width);
	  balloons.velocity[i].x *= -1;
	}
	if (balloons.point[i].y < 0) {
	  balloons.point[i].y *= -1;
	  balloons.velocity[i].y *= -1;
	} else if (balloons.point[i].y + xballoon.balloon_height >
		   xballoon.height) {
	  balloons.point[i].y -=
	    (balloons.point[i].y + xballoon.balloon_height - xballoon.height);
	  balloons.velocity[i].y *= -1;
	}
      }
    }
    for (i = 0; i < balloons.number; i++) {
      DrawPixmap1(balloons.point_old[i], balloons.point[i]);
      XFlush(xballoon.display);
      usleep(1000);
    }
    usleep(xballoon.wait);
  }
}

main(int argc, char ** argv)
{
  SetDefaultValue();
  ReadParameters(&argc, argv);
  InitializeDisplay();
  InitializeBalloon();
  InitializeSignal();
  BalloonMain();
  BalloonQuit();
}

/*****************************************************************************/
/* End of XBalloon.                                                          */
/*****************************************************************************/
